<?php

namespace Drupal\open_ai_metadata\Form;

use Drupal\Core\Form\ConfigFormBase;
use Drupal\Core\Form\FormStateInterface;
use Drupal\Core\Entity\EntityTypeManagerInterface;
use Symfony\Component\DependencyInjection\ContainerInterface;
use Drupal\Core\Ajax\AjaxResponse;
use Drupal\Core\Ajax\InvokeCommand;
use Drupal\Core\Ajax\CloseDialogCommand;
use GuzzleHttp\Client;
use GuzzleHttp\Exception\GuzzleException;

/**
 * Stores metadata content type settings.
 */
class MetadataContentForm extends ConfigFormBase {

  /**
   * The entity type manager service.
   *
   * @var \Drupal\Core\Entity\EntityTypeManagerInterface
   */
  protected $entityTypeManager;

  /**
   * {@inheritdoc}
   */
  protected function getEditableConfigNames() {
    return [
      'open_ai_metadata.content_settings',
    ];
  }

  /**
   * Constructs a MetadataConfigForm object.
   *
   * @param \Drupal\Core\Entity\EntityTypeManagerInterface $entityTypeManager
   *   The entity type manager service.
   */
  public function __construct(EntityTypeManagerInterface $entityTypeManager) {
    $this->entityTypeManager = $entityTypeManager;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container) {
    return new static(
      $container->get('entity_type.manager')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function getFormId() {
    return 'metadata_content_form';
  }

  /**
   * {@inheritdoc}
   */
  public function buildForm(array $form, FormStateInterface $form_state, $fieldName = NULL) {

    $form['notification_instructions'] = [
      '#type' => 'fieldset',
      '#title' => $this->t('Instructions'),
      '#markup' => $this->t('Generate content based on your title for body'),
    ];

    $content_types = $this->entityTypeManager->getStorage('node_type')->loadMultiple();

    $options = [];
    foreach ($content_types as $content_type) {
      $options[$content_type->id()] = $content_type->label();
    }

    $form['#attached']['library'][] = 'open_ai_metadata/open_ai_metadata';
    $form['#attached']['library'][] = 'core/ckeditor5';
    $form['metadata_content_title'] = [
      '#type' => 'textfield',
      '#tag' => 'input',
      '#title' => $this->t('Content Title'),
      '#placeholder' => $this->t('enter your content title . . .'),
      '#attributes' => [
        'id' => 'edit-metadata-content-title',
      ],
    ];

    $form['metadata_content_body'] = [
      '#type' => 'textarea',
      '#title' => $this->t('Generated Content'),
      '#placeholder' => $this->t('Your content will be generated here . . .'),
      '#attributes' => [
        'id' => 'metadata-content-body',
      ],
    ];

    $form['metadata_content_generate'] = [
      '#type' => 'button',
      '#value' => $this->t('Generate Content'),
      '#attributes' => [
        'style' => 'background-color: lightblue;',
      ],
      '#ajax' => [
        'callback' => '::generateBodyContent',
        'event' => 'click',
      ],
    ];

    $form['metadata_content_use'] = [
      '#type' => 'button',
      '#value' => $this->t('Use Content'),
      '#attributes' => [
        'style' => ['background-color: lightgreen;'],
        'class' => ['use-content-button'],
        'name' => $fieldName,
      ],
      '#ajax' => [
        'callback' => '::useContentForBody',
        'event' => 'click',
      ],
    ];

    return $form;
  }

  /**
   * {@inheritdoc}
   */
  public function validateForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * {@inheritdoc}
   */
  public function submitForm(array &$form, FormStateInterface $form_state) {

  }

  /**
   * Generate Body Content.
   */
  public function generateBodyContent(array &$form, FormStateInterface $form_state) {

    $logger = \Drupal::logger('open_ai_metadata');
    $input = $form['metadata_content_title']['#value'];
    $res = $this->generateContent($input);
    $response = new AjaxResponse();

    if ($res && !empty($res)) {
      $response->addCommand(new InvokeCommand('#metadata-content-body', 'val', [$res]));
    }
    else {
      $logger->error('No Content Found');
    }

    return $response;
  }

  /**
   * Function to use the content generated in the body field.
   */
  public function useContentForBody(array &$form, FormStateInterface $form_state) {

    $response = new AjaxResponse();
    $response->addCommand(new CloseDialogCommand());

    return $response;
  }

  /**
   * Function to generate content from Open AI.
   */
  public function generateContent($value) {
    $config = \Drupal::config('open_ai_metadata.settings');
    $state_config = \Drupal::state();

    $api_endpoint = $config->get('open_ai_metadata.openai_endpoint');
    $access_token = $state_config->get('open_ai_metadata.openai_token');
    $max_tokens = (int) $config->get('open_ai_metadata.openai_max_token');
    $model = $config->get('open_ai_metadata.openai_model');
    $temperature = (float) $config->get('open_ai_metadata.openai_temperature');

    $answer = "";
    $logger = \Drupal::logger('open_ai_metadata');
    $title = trim($value);
    $prompt = "Generate body content for the title : '" . $title . "' ";

    if ($api_endpoint && $access_token && $temperature && $max_tokens && $model) {
      $message[] = [
        "role" => "user",
        "content" => $prompt,
      ];

      $api_request_body = [
        'model' => $model,
        'messages' => $message,
        'temperature' => $temperature,
        'max_tokens' => $max_tokens,
      ];

      $client_config = [
        'headers' => [
          'Content-Type' => 'application/json',
          'Authorization' => 'Bearer ' . $access_token,
        ],
        'body' => json_encode($api_request_body),
      ];

      try {

        if ($title) {
          // Guzzle HTTP client instance.
          $client = new Client();

          // Guzzle HTTP POST request to the API endpoint.
          $response = $client->post($api_endpoint, $client_config);

          $response_body = json_decode($response->getBody());
          $answer = $response_body ? $response_body->choices[0]->message->content : "";
        }
        else {
          $logger->error('No title received!');
        }

      }
      catch (GuzzleException $exception) {
        $errorMessage = $exception->getMessage();
        $logger->error($errorMessage);
      }
    }
    else {
      $logger->error('Incomplete Open AI credentials.');
    }

    if ($answer && !empty($answer)) {
      return $answer;
    }

  }

}
